@@ -61,7 +61,7 @@ def explain(include: InclusionChoice = "explain") -> None:
 
 
 def friendly_tb() -> None:
-    """Shows the a simplified Python traceback,
+    """Shows the simplified Python traceback,
     which includes the hint/suggestion if available.
     """
     explain("friendly_tb")
@@ -143,7 +143,10 @@ def why() -> None:
     # If no cause is found, and the exception name is not accompanied by a
     # message, as in "StopIteration:", we use the same info for
     # the cause as we used for the generic information as per issue #66
-    info = session.saved_info[-1]
+    try:
+        info = session.saved_info[-1]
+    except IndexError:
+        info = {}
     if (
         ("cause" not in info or not info["cause"])
         and "message" in info
@@ -303,6 +306,11 @@ def _get_tb_data() -> Optional[TracebackData]:  # pragma: no cover
     return info["_tb_data"]
 
 
+def _get_info() -> list:
+    """Debugging tool: returns the content of a traceback info."""
+    return session.saved_info[-1] if session.saved_info else []
+
+
 def _show_info() -> None:  # pragma: no cover
     """Debugging tool: shows the complete content of traceback info.
 
@@ -354,6 +362,7 @@ add_help_attribute(helpers)
 debug_helper_methods: Dict[str, Callable[..., Any]] = {
     "_debug_tb": _debug_tb,
     "_get_frame": _get_frame,
+    "_get_info": _get_info,
     "_show_info": _show_info,
     "_get_tb_data": _get_tb_data,
     "_get_exception": _get_exception,
@@ -376,7 +385,6 @@ class FriendlyHelpers:
 
     def __init__(self) -> None:
         self.helpers = {}
-        self.__class__.__name__ = "Friendly"  # For a nicer Rich repr
 
     def add_helper(self, function: Callable) -> None:  # pragma: no cover
         """Adds a helper base on its name and the function it refers to."""
@@ -389,7 +397,8 @@ class FriendlyHelpers:
             del self.helpers[name]
             delattr(self, name)
         else:
-            debug_helper.log(f"Cannot remove {name}; it is not a known helper.")
+            message = f"Cannot remove {name}; it is not a known helper."
+            debug_helper.log(message)
 
     def __dir__(self) -> List[str]:  # pragma: no cover
         """Only include useful friendly methods."""
@@ -419,7 +428,7 @@ class FriendlyHelpers:
             "The following methods of the Friendly object should also "
             "be available as functions."
         )
-        parts = [header + "\n\n"]
+        parts = [self.true_repr() + "\n" + header + "\n\n"]
         for name in basic_helpers:
             parts.append(name + "(): ")
             fn = self.helpers[name]
@@ -440,6 +449,10 @@ class FriendlyHelpers:
                     print("Warning:", name, "has no help() method.")
         return "".join(parts)
 
+    def true_repr(self) -> str:  # pragma: no cover
+        """Method that can be called when a normal looking repr is needed."""
+        return "<class FriendlyHelpers>"
+
 
 Friendly = FriendlyHelpers()
 for helper_name in helpers:
