@@ -7,15 +7,17 @@ they are considered to be internal functions, subject to change at any
 time. If functions defined in friendly_traceback.__init__.py do not meet your needs,
 please file an issue.
 """
-import ast
+import contextlib
 import inspect
+import io
 import re
+import sys
 import traceback
 import types
 from itertools import dropwhile
 from typing import List, Optional, Sequence, Tuple, Type
 
-from . import debug_helper, info_generic, info_specific, info_variables, token_utils
+from . import debug_helper, info_generic, info_specific, info_variables
 from .frame_info import FrameInfo
 from .ft_gettext import current_lang
 from .path_info import is_excluded_file, path_utils
@@ -36,7 +38,7 @@ _ = current_lang.translate
 def convert_value_to_message(value: BaseException) -> str:
     """This converts the 'value' of an exception into a string, while
     being safe to use for custom exceptions which have been incorrectly
-    defined. See issue #181 for an example.
+    defined. See https://github.com/aroberge/friendly/issues/181 for an example.
     """
     try:
         message = str(value)
@@ -45,6 +47,23 @@ def convert_value_to_message(value: BaseException) -> str:
     return message
 
 
+def retrieve_message(etype: Type[_E], value: _E, tb: types.TracebackType) -> str:
+    "Safely retrieves the message, including any additional hint from Python."
+    message = convert_value_to_message(value)
+    if (
+        message == STR_FAILED
+        or sys.version_info < (3, 10)
+        or etype not in (AttributeError, NameError)
+    ):
+        return message
+    # 3.10+ hints are not directly accessible from Python.
+    err = io.StringIO()
+    with contextlib.redirect_stderr(err):
+        sys.__excepthook__(etype, value, tb)
+    full_message = err.getvalue().split("\n")[-2]
+    return full_message.split(":", 1)[1].strip()
+
+
 class TracebackData:
     """Raw traceback info obtained from Python.
 
@@ -63,14 +82,10 @@ class TracebackData:
         self.exception_type = etype
         self.exception_name = etype.__name__
         self.value = value
-        self.message = convert_value_to_message(value)
-        if isinstance(tb, str):  # for SyntaxErrors from IDLE hack
-            self.formatted_tb = tb
-            self.records = []
-        else:
-            self.formatted_tb = traceback.format_exception(etype, value, tb)
-            self.records = self.get_records(tb)
-            self.python_records = self.get_records(tb, python_excluded=False)
+        self.message = retrieve_message(etype, value, tb)
+        self.formatted_tb = traceback.format_exception(etype, value, tb)
+        self.records = self.get_records(tb)
+        self.python_records = self.get_records(tb, python_excluded=False)
 
         # The following three attributes get their correct values in get_source_info()
         self.bad_line = "\n"
@@ -212,7 +227,7 @@ class TracebackData:
             self.exception_frame = record.frame
             self.filename = record.filename
             line = record.problem_line()
-            self.bad_line = line.rstrip()
+            self.bad_line = line.strip()  # fix for 3.11
             # protecting against https://github.com/alexmojaki/stack_data/issues/13
             if not self.bad_line:
                 try:
@@ -264,89 +279,13 @@ class TracebackData:
                 self.original_bad_line = self.bad_line
                 self.bad_line = self.node_text
 
-    @staticmethod
-    def find_tb_frame(
-        tb: types.TracebackType, frame: types.FrameType
-    ) -> Optional[types.TracebackType]:
-        """Find a traceback object where a given frame resides."""
-        while True:
-            if tb.tb_frame == frame:
-                return tb
-            tb = tb.tb_next
-            if not tb:  # pragma: no cover
-                debug_helper.log("No tb_frame found.")
-                return None
-
-    def locate_name_error(self) -> None:
-        """Finds the location of an unknown name"""
-        parts = self.message.split("'")
-        if len(parts) < 2:
-            debug_helper.log("Could not extract name from NameError.message.")
-            return
-        name = parts[1]
-
-        if name and name in self.bad_line:
-            begin = self.bad_line.find(name)
-            end = begin + len(name)
-            self.node_range = begin, end
-        else:  # pragma: no cover
-            debug_helper.log("Could not locate unknown name.")
-
-    @staticmethod
-    def find_node(
-        tb: types.TracebackType, bad_line: str
-    ) -> Optional[Tuple[ast.AST, Optional[Tuple[int, int]], str]]:
-        """Finds the 'node', that is the exact part of a line of code
-        that is related to the cause of the problem.
-        """
-        try:
-            ex: executing.Executing = executing.Source.executing(tb)
-            node = ex.node
-            node_text = ex.text()
-        except Exception as e:  # pragma: no cover
-            debug_helper.log("Exception raised in TracebackData.use_executing.")
-            debug_helper.log(str(e))
-            return
-        # If we can find the precise location (node) on a line of code
-        # causing the exception, we note this location
-        # so that we can indicate it later with ^^^^^, something like:
-        #
-        #    20:     b = tuple(range(50))
-        #    21:     try:
-        # -->22:         print(a[50], b[0])
-        #                      ^^^^^
-        #    23:     except Exception as e:
-        #
-        # If the node spans the entire line, we do not bother to indicate
-        # its specific location.
-        #
-        # Sometimes, a node will span multiple lines. For example,
-        # line 22 shown above might have been written as:
-        #
-        #    print(a[
-        #            50], b[0])
-        #
-        # If that is the case, we rewrite the node as a single line.
-
-        # To start, we transform logical line (or parts thereof) into
-        # something that fits on a single physical line.
-        # \n could be a valid newline token or a character within
-        # a string; we only want to replace newline tokens.
-        node_range = None
-        if "\n" in node_text:
-            tokens = token_utils.tokenize(node_text)
-            tokens = [tok for tok in tokens if tok != "\n"]
-            node_text = "".join(tok.string for tok in tokens)
-        bad_code = token_utils.strip_comment(bad_line)
-        if (
-            node_text
-            and node_text in bad_line
-            and node_text.strip() != bad_code.strip()
-        ):
-            begin = bad_line.find(node_text)
-            end = begin + len(node_text)
-            node_range = begin, end
-        return node, node_range, node_text
+        # Also attempt to restrict the information about where the program
+        # stopped to the strict minimum so that we don't show irrelevant
+        # values of names
+        if self.records[0].node_info and self.records[0].node_info != node_info:
+            node, _ignore, node_text = self.records[0].node_info
+            if node_text.strip():
+                self.program_stopped_bad_line = node_text
 
 
 # ====================
@@ -370,9 +309,9 @@ class TracebackData:
 
 # [cause_header and cause]
 # Likely cause:
-#     The variable that appears to cause the problem is 'a'.
+#     The variable that appears to cause the problem is 'abc'.
 #     Try inserting the statement
-#         global a
+#         global abc
 #     as the first line inside your function.
 #
 
@@ -402,8 +341,8 @@ class FriendlyTraceback:
     in the docstrings of the relevant methods.
 
     To get all possible attributes set up, one needs to call
-    compile_info() after initializing this class. This is done so
-    as to allow third-party users to selectively call only one of
+    compile_info() after initializing this class. This is done
+    to allow third-party users to selectively call only one of
 
     * assign_cause()
     * assign_generic()
@@ -434,12 +373,11 @@ class FriendlyTraceback:
             self.tb_data = TracebackData(etype, value, tb)
         except Exception as e:  # pragma: no cover
             debug_helper.log("Uncaught exception in TracebackData:")
-            print(f"Internal error caused by {e!r}")
-            debug_helper.handle_internal_error()
+            debug_helper.handle_internal_error(e)
             raise SystemExit
         self.suppressed = ["       ... " + _("More lines not shown.") + " ..."]
         self.info = {"header": _("Python exception:")}
-        self.message = self.assign_message()  # language independent
+        self.message = self.assign_message(etype, value, tb)  # language independent
         self.assign_tracebacks()
 
         # include some values for debugging purpose in an interactive session
@@ -447,27 +385,27 @@ class FriendlyTraceback:
         self.info["_frame"] = self.tb_data.exception_frame
         self.info["_tb_data"] = self.tb_data
 
-    def assign_message(self) -> str:
+    def assign_message(self, etype, value, tb) -> str:
         """Assigns the error message, as the attribute ``message``
         which is something like::
 
             NameError: name 'a' is not defined
         """
-        exc_name = self.tb_data.exception_name
-
-        value = self.tb_data.value
+        exc_name = etype.__name__
         if hasattr(value, "msg"):
             self.info["message"] = f"{exc_name}: {value.msg}\n"
         else:
-            message = convert_value_to_message(value)
+            message = retrieve_message(etype, value, tb)
             self.info["message"] = f"{exc_name}: {message}\n"
         return self.info["message"]
 
     def compile_info(self) -> None:
         """Compile all info that was not set in __init__."""
         self.assign_generic()
-        self.assign_location()
+        # For SyntaxError, assigning the cause may result in better location
+        # information; so we need to do this first.
         self.assign_cause()
+        self.assign_location()
         # removing null values; mypy cannot figure out the type correctly here
         to_remove = [key for key in self.info if not self.info[key]]  # type: ignore
         for key in to_remove:
@@ -485,7 +423,11 @@ class FriendlyTraceback:
         """Determine the cause of an exception, which is what is returned
         by ``why()``.
         """
-        if self.tb_data.filename in ["<unknown>", "<string>"]:
+        if self.tb_data.filename == "<unknown>" or (
+            self.tb_data.filename == "<string>"
+            and self.tb_data.value.lineno != 1
+            and not issubclass(self.tb_data.exception_type, SyntaxError)
+        ):
             return
 
         if STR_FAILED in self.message:
@@ -521,8 +463,6 @@ class FriendlyTraceback:
             etype, value, self.tb_data.exception_frame, self.tb_data
         )  # [3]
         self.info.update(**cause)
-        if self.tb_data.filename == "<stdin>":  # pragma: no cover
-            self.info["cause"] += "\n" + cannot_analyze_stdin()
 
     def set_cause_syntax(self) -> None:
         """For SyntaxError and subclasses. Sets the value of the following
@@ -543,10 +483,6 @@ class FriendlyTraceback:
         if self.tb_data.filename == "<unknown>":
             return
 
-        if self.tb_data.filename == "<stdin>":  # pragma: no cover
-            self.info["cause"] = cannot_analyze_stdin()
-            return
-
         if "encoding problem" in str(self.tb_data.value):
             self.info["cause"] = _("The encoding of the file was not valid.\n")
             return
@@ -599,8 +535,10 @@ class FriendlyTraceback:
             return
 
         self.locate_exception_raised(records[-1])
+        self.info["detailed_tb"] = self.get_detailed_stack_info(records)
         if len(records) > 1:
-            self.locate_last_call(records[0])
+            _ignore, partial_source, var_info = self.info["detailed_tb"][0]
+            self.locate_last_call(records[0], partial_source, var_info)
 
     def locate_exception_raised(self, record: FrameInfo) -> None:
         """Sets the values of the following attributes which are
@@ -612,6 +550,11 @@ class FriendlyTraceback:
         """
         from .config import session
 
+        if self.tb_data.filename == "<stdin>":
+            self.info["exception_raised_source"] = cannot_analyze_stdin()
+            self.info["exception_raised_header"] = ""
+            return
+
         partial_source = record.partial_source_with_node_range
         if partial_source["source"].strip() == "0:":
             partial_source["source"] = ""
@@ -646,14 +589,14 @@ class FriendlyTraceback:
         if var_info:
             self.info["exception_raised_variables"] = var_info
 
-    def locate_last_call(self, record: FrameInfo) -> None:
+    def locate_last_call(self, record: FrameInfo, partial_source, var_info) -> None:
         """Sets the values of the following attributes:
 
         * last_call_header
         * exception_raised_source
         * last_call_variables
         """
-        partial_source = record.partial_source_with_node_range
+        # partial_source = record.partial_source_with_node_range
         filename = path_utils.shorten_path(record.filename)
 
         if filename and "[" in filename:
@@ -664,12 +607,41 @@ class FriendlyTraceback:
             self.info["last_call_header"] = _(
                 "Execution stopped on line {linenumber} of file {filename}.\n"
             ).format(linenumber=record.lineno, filename=filename)
-        self.info["last_call_source"] = partial_source["source"]
+        # self.info["last_call_source"] = partial_source["source"]
+        self.info["last_call_source"] = partial_source
 
-        var_info = info_variables.get_var_info(partial_source["line"], record.frame)
+        # var_info = info_variables.get_var_info(
+        #     self.tb_data.program_stopped_bad_line, record.frame
+        # )
         if var_info:
             self.info["last_call_variables"] = var_info
 
+    def get_detailed_stack_info(self, records):
+        # sourcery skip: use-named-expression
+        if self.tb_data.filename == "<stdin>":
+            return []
+
+        detailed_tb = []
+        for record in self.tb_data.records:
+            filename = path_utils.shorten_path(record.filename)
+            lineno = record.lineno
+            if record.node_info:
+                _node, _ignore, line = record.node_info
+            else:
+                line = record.problem_line()
+            partial_source = record.partial_source_with_node_range
+            var_info = info_variables.get_var_info(line, record.frame)
+            if "[" in filename:
+                location = _("Code block {filename}, line {line}").format(
+                    filename=filename, line=lineno
+                )
+            else:
+                location = _("File {filename}, line {line}").format(
+                    filename=filename, line=lineno
+                )
+            detailed_tb.append((location, partial_source["source"], var_info))
+        return detailed_tb
+
     def locate_parsing_error(self) -> None:
         """Sets the values of the attributes:
 
@@ -678,11 +650,11 @@ class FriendlyTraceback:
         """
         value = self.tb_data.value
         filepath = value.filename
-        if filepath in ["<unknown>", "<string>"]:
-            self.info["parsing_error"] = _(
-                "`{filename}` is not a regular Python file whose contents\n"
-                "can be analyzed.\n"
-            ).format(filename=filepath)
+        not_regular_file = _(
+            "The entire content of `{filename}` is not available.\n"
+        ).format(filename=filepath)
+        if filepath == "<unknown>":
+            self.info["parsing_error"] = not_regular_file
             return
 
         statement = self.tb_data.statement
@@ -700,10 +672,12 @@ class FriendlyTraceback:
                 "Python could not understand the code in the file\n'{filename}'\n"
             ).format(filename=short_filename)
 
-        if "-->" in partial_source and "^" in partial_source:
+        if "  ^" in partial_source:
             self.info["parsing_error"] = could_not_understand + _(
-                "at the location indicated by ^.\n"
+                "at the location indicated.\n"
             ).format(filename=short_filename)
+            if filepath in ["<string>", "<stdin>"] and self.tb_data.value.lineno != 1:
+                self.info["parsing_error"] += not_regular_file
         elif filepath:  # could be None
             self.info["parsing_error"] = could_not_understand + ".\n"
 
@@ -721,7 +695,7 @@ class FriendlyTraceback:
            project.  In addition, for RecursionError, this traceback is often
            further shortened, compared with a normal Python traceback.
         3. A potentially shortened traceback, which does not include too much
-           output so as not to overwhelm beginners. It also include information
+           output so as not to overwhelm beginners. It also includes information
            about the code on any line mentioned.
 
         These are given by the attributes:
@@ -775,7 +749,7 @@ class FriendlyTraceback:
         chain_info = ""
         short_chain_info = ""
         if exc.__cause__ or exc.__context__:
-            chain_info = self.process_exception_chain(self.tb_data.exception_type, exc)
+            chain_info = process_exception_chain(self.tb_data.exception_type, exc)
             parts = chain_info.split("\n\n")
             # suppress line
             temp = []
@@ -821,44 +795,6 @@ class FriendlyTraceback:
             temp.append(line)
         return temp
 
-    @staticmethod
-    def process_exception_chain(etype: Type[_E], value: _E) -> str:
-        """Adds info about exceptions raised while treating other exceptions."""
-        seen = set()
-        lines = []
-
-        direct_cause = (
-            "The above exception was the direct cause of the following exception:"
-        )
-        another_exception = (
-            "During handling of the above exception, another exception occurred:"
-        )
-
-        def chain_exc(typ: Type[_E], exc: _E, tb: Optional[types.TracebackType]) -> str:
-            seen.add(id(exc))
-            context = exc.__context__
-            cause = exc.__cause__
-            if cause is not None and id(cause) not in seen:
-                chain_exc(type(cause), cause, cause.__traceback__)
-                lines.append(f"\n    {direct_cause}\n\n")
-            elif (
-                context is not None
-                and not exc.__suppress_context__
-                and id(context) not in seen
-            ):
-                chain_exc(type(context), context, context.__traceback__)
-                lines.append(f"\n    {another_exception}\n\n")
-            if tb:
-                tbe = traceback.extract_tb(tb)
-                lines.append("Traceback (most recent call last):\n")
-                for line in traceback.format_list(tbe):
-                    lines.append(line)
-                for line in traceback.format_exception_only(typ, exc):
-                    lines.append(line)
-
-        chain_exc(etype, value, None)
-        return "".join(lines)
-
     def create_traceback(self, records: List[FrameInfo]) -> List[str]:
         """Using records that exclude code from certain files,
         creates a list from which a standard-looking traceback can
@@ -924,6 +860,45 @@ class FriendlyTraceback:
         return result
 
 
+def process_exception_chain(etype: Type[_E], value: _E) -> str:
+    """Obtains info about exceptions raised while treating other exceptions."""
+    seen = set()
+    lines = []
+
+    direct_cause = (
+        "The above exception was the direct cause of the following exception:"
+    )
+    another_exception = (
+        "During handling of the above exception, another exception occurred:"
+    )
+
+    def chain_exc(typ: Type[_E], exc: _E, tb: Optional[types.TracebackType]) -> None:
+        """Recursive function that insert the contents of 'lines' above."""
+        seen.add(id(exc))
+        context = exc.__context__
+        cause = exc.__cause__
+        if cause is not None and id(cause) not in seen:
+            chain_exc(type(cause), cause, cause.__traceback__)
+            lines.append(f"\n    {direct_cause}\n\n")
+        elif (
+            context is not None
+            and not exc.__suppress_context__
+            and id(context) not in seen
+        ):
+            chain_exc(type(context), context, context.__traceback__)
+            lines.append(f"\n    {another_exception}\n\n")
+        if tb:
+            tbe = traceback.extract_tb(tb)
+            lines.append("Traceback (most recent call last):\n")
+            for line in traceback.format_list(tbe):
+                lines.append(line)
+            for line in traceback.format_exception_only(typ, exc):
+                lines.append(line)
+
+    chain_exc(etype, value, None)
+    return "".join(lines)
+
+
 def cannot_analyze_stdin() -> str:  # pragma: no cover
     """Typical case: friendly is imported in an ordinary Python
     interpreter (REPL), and the user does not activate the friendly
