@@ -109,19 +109,50 @@ class FrameInfo(stack_data.FrameInfo):
         new_lines = []
         problem_line = ""
         nb_digits = len(str(lines[-1].lineno))
-        no_mark = "       {:%d}: " % nb_digits
-        with_mark = "    -->{:%d}: " % nb_digits
+        # fmt: off
+        no_mark       = "       {:%d}: " % nb_digits  # noqa
+        with_mark     = "    -->{:%d}: " % nb_digits  # noqa
+        line_gap_mark = "      (...)"
+        # fmt: on
 
         text_range_mark = None
-        if with_node_range and self.node_info and self.node_info[1]:
-            begin, end = self.node_info[1]
-            text_range_mark = " " * (8 + nb_digits + begin + 1) + "^" * (end - begin)
+        if with_node_range and self.node_info:
+            if self.node_info[1]:
+                begin, end = self.node_info[1]
+                text_range_mark = " " * (8 + nb_digits + begin + 1) + "^" * (
+                    end - begin
+                )
+            elif self.node_info[2] and isinstance(self.node_info[2], str):
+                text = self.node_info[2]
+                for line_obj in lines:
+                    if line_obj is stack_data.LINE_GAP:
+                        continue
+                    if line_obj.is_current:
+                        begin = line_obj.text.find(text)
+                        if begin >= 0:
+                            text_range_mark = " " * (
+                                8 + nb_digits + begin + 1
+                            ) + "^" * len(text)
 
         marked = False
+        prev_lineno = None
+        # stack_data does not include empty lines. However, I believe that
+        # this might not be helpful for beginners who look at the code
+        # without paying too much attention at the line numbers.
+        # I fixed this below.
         for line_obj in lines:
             if line_obj is stack_data.LINE_GAP:
-                new_lines.append(" " * 7 + "(...)")
-            elif line_obj.is_current:
+                new_lines.append(line_gap_mark)
+                prev_lineno = None
+                continue
+            if prev_lineno and line_obj.lineno - prev_lineno == 2:
+                # add single empty line
+                num = no_mark.format(prev_lineno + 1)
+                new_lines.append(num)
+            if prev_lineno and line_obj.lineno - prev_lineno > 2:
+                # add line gap to indicate that some empty lines were skipped
+                new_lines.append(line_gap_mark)
+            if line_obj.is_current:
                 num = with_mark.format(line_obj.lineno)
                 problem_line = line_obj.text
                 new_lines.append(num + problem_line.rstrip())
@@ -136,6 +167,8 @@ class FrameInfo(stack_data.FrameInfo):
             else:
                 num = no_mark.format(line_obj.lineno)
                 new_lines.append(num + line_obj.text.rstrip())
+            prev_lineno = line_obj.lineno
+
         return "\n".join(new_lines), problem_line
 
     @cached_property
@@ -171,8 +204,12 @@ class FrameInfo(stack_data.FrameInfo):
         #            50], b[0])
         #
         # If that is the case, we rewrite the node as a single line.
+        special_case = False
         if not node_text:
-            return
+            node_text = self.handle_special_cases()
+            special_case = True
+            if not node_text:
+                return
 
         node_range = None
         if "\n" in node_text:
@@ -193,6 +230,9 @@ class FrameInfo(stack_data.FrameInfo):
             begin = bad_line.find(node_text)
             end = begin + len(node_text)
             node_range = begin, end
+        if special_case:  # use the entire bad_line to determine the cause
+            # use node_range to highlight.
+            return node, node_range, bad_line
         return node, node_range, node_text
 
     @cached_property
@@ -201,6 +241,29 @@ class FrameInfo(stack_data.FrameInfo):
             line for line in self.lines if isinstance(line, Line) and line.is_current
         )
 
+    def handle_special_cases(self):
+        """Hack to try to identify a problematic text when node_text is None.
+
+        We just use the information to highlight where in a statement
+        the error is located.
+        """
+        try:
+            exec(self.current_line.text)
+        except Exception as exc:
+            saved_exc = exc
+        else:
+            return ""
+
+        if not hasattr(saved_exc, "msg"):
+            return ""
+        message = saved_exc.msg
+        if message.startswith("cannot import name '"):
+            return message.split("'")[1]
+        elif message.startswith("No module named '"):
+            return message.split("'")[1]
+
+        return ""
+
 
 class FakeLineObject:
     """Class reproducing the minimum attributes for formatting lines"""
