@@ -45,6 +45,7 @@ def convert_type(short_form: str) -> str:
 
 
 def get_all_objects(line: str, frame: types.FrameType) -> ObjectsInfo:
+    # sourcery skip: assign-if-exp, simplify-generator
     """Given a (partial) line of code and a frame,
     obtains a dict containing all the relevant information about objects
     found on that line so that they can be formatted as part of the
@@ -87,7 +88,12 @@ def get_all_objects(line: str, frame: types.FrameType) -> ObjectsInfo:
                 if name in scope_dict:
                     names.add(name)
                     obj = scope_dict[name]
-                    objects[scope].append((name, repr(obj), obj))
+                    if hasattr(obj, "true_repr"):  # sourcery: skip
+                        # guard against the case where obj == Friendly; #106
+                        repr_obj = obj.true_repr()
+                    else:
+                        repr_obj = repr(obj)
+                    objects[scope].append((name, repr_obj, obj))
                     objects["name, obj"].append((name, obj))
                     break
             else:
@@ -104,25 +110,29 @@ def get_all_objects(line: str, frame: types.FrameType) -> ObjectsInfo:
         atok = ASTTokens(line.strip(), parse=True)
     except SyntaxError as e:
         if "unexpected EOF" not in str(e):
-            debug_helper.log(f"problem with ASTTokens: {e}")
-            debug_helper.log(f"line: {line}")
+            debug_helper.log(f"Problem with ASTTokens: {e}" + f"\nline: {line}")
         return objects
 
     if atok is not None:
         evaluator = Evaluator.from_frame(frame)
-        for nodes, obj in group_expressions(
-            pair for pair in evaluator.find_expressions(atok.tree)
-        ):
-            name = atok.get_text(nodes[0])
-            if not name or name in names:
-                continue
-            names.add(name)
-            objects["name, obj"].append((name, obj))
-            try:
-                # We're not interested in showing literals in the list of variables
-                ast.literal_eval(name)
-            except Exception:  # noqa
-                objects["expressions"].append((name, obj))
+        try:
+            for nodes, obj in group_expressions(
+                pair for pair in evaluator.find_expressions(atok.tree)
+            ):
+                name = atok.get_text(nodes[0])
+                if not name or name in names:
+                    continue
+                names.add(name)
+                objects["name, obj"].append((name, obj))
+                try:
+                    # We're not interested in showing literals in the list of variables
+                    ast.literal_eval(name)
+                except Exception:  # noqa
+                    objects["expressions"].append((name, obj))
+        except Exception:  # noqa
+            # The example in https://github.com/ipython/ipython/issues/13481
+            # give rises to a TypeError exception here.
+            pass
 
     return objects
 
@@ -250,15 +260,12 @@ def simplify_repr(name: str, splitlines: bool = True) -> str:
     Does additional formatting in an attempt to make the names (repr)
     more readable.
     """
-    if not name.startswith("<"):
-        debug_helper.log("simplify_repr called on name that does not start with <")
-    if name.endswith(">>"):
-        end_angle = ">>"
-    elif name.endswith(">"):
-        end_angle = ">"
-    else:
-        debug_helper.log(f"in simplify_repr, unexpected ending of {name}")
-        end_angle = ""
+    if not name.startswith("<") or not name.endswith(">"):
+        debug_helper.log("simplify_repr called on name that is not of the form <...>")
+        debug_helper.log(f"name={name}")
+        return name
+    end_angle = ">>" if name.endswith(">>") else ">"
+
     bound_method = "bound method" in name
     if " at " in name:
         # There are two reasons to remove the memory location information:
@@ -319,7 +326,12 @@ def simplify_bound_method(name: str, splitlines: bool = False) -> str:
         obj_name = obj_parts[-1]
         file_name = ".<locals>".join(obj_parts[0:-1])
         name = (
-            method + " of <" + obj_name + " defined in <function " + file_name[1:] + ">"
+            method
+            + " of <"
+            + obj_name
+            + "` defined in `<function "
+            + file_name[1:]
+            + ">"
         )
         if len(name) > MAX_LENGTH and splitlines:
             of_object = f"\n{INDENT}of <" + obj_name
@@ -374,8 +386,12 @@ def format_var_info(name: str, value: str, obj: str, _global: str = "") -> str:
             value = value[0 : MAX_LENGTH - 5] + "..." + value[-1]
         try:
             length_info = len(obj)
+        except OverflowError:
+            length_info = _("Object too large to be processed by Python.")
         except TypeError:
             pass
+        except Exception as e:
+            length_info = _("Unable to compute.") + f" ({e.__class__.__name__})"
 
     result = f"    {_global}{name}:  {value}"
     if length_info:
