@@ -9,9 +9,10 @@ import ast
 import re
 import sys
 
-from .. import debug_helper, utils
+from .. import debug_helper, token_utils, utils
 from ..ft_gettext import current_lang, please_report
-from . import error_in_def, fixers, statement_analyzer, syntax_utils
+from . import error_in_def, fixers, statement_analyzer
+from . import syntax_utils as su
 
 MESSAGE_ANALYZERS = []
 _ = current_lang.translate
@@ -32,6 +33,8 @@ def _find_keyword(statement):
     # used in assign_to_keyword
     if statement.bad_token.is_keyword():
         return statement.bad_token
+    elif statement.prev_token.is_keyword():
+        return statement.prev_token
     else:  # something like name.constant = ?
         for tok in statement.tokens[statement.bad_token_index :]:
             if tok.is_keyword():
@@ -45,7 +48,7 @@ def _what_kind_of_literal(literal):
     try:
         a = ast.literal_eval(literal)
     except Exception:  # noqa
-        return None
+        return ""
 
     kinds = (
         (int, _("of type `int`")),
@@ -62,7 +65,7 @@ def _what_kind_of_literal(literal):
             return result
 
     debug_helper.log("New kind of literal" + str(a))  # pragma: no cover
-    return None  # pragma: no cover
+    return ""  # pragma: no cover
 
 
 def _proper_decimal_or_octal_number(prev_str, bad_str):
@@ -96,7 +99,7 @@ def _proper_decimal_or_octal_number(prev_str, bad_str):
 
 
 def add_python_message(func):
-    """A simple decorator that adds a function the the list of functions
+    """A simple decorator that adds a function the list of functions
     that process a message given by Python.
     """
     MESSAGE_ANALYZERS.append(func)
@@ -118,7 +121,7 @@ def analyze_message(message: str = "", statement=None):
 
 
 @add_python_message
-def assign_to_conditional_expression(message: str = "", _statement=None):
+def assign_to_conditional_expression(message: str = "", statement=None):
     if message not in (
         "can't assign to conditional expression",  # Python 3.6, 3.7
         "cannot assign to conditional expression",  # Python 3.8
@@ -131,6 +134,28 @@ def assign_to_conditional_expression(message: str = "", _statement=None):
         "A conditional expression has the following form:\n\n"
         "    variable = object if condition else other_object\n"
     )
+
+    expression = su.get_expression_before_specified_token(
+        statement.bad_token, statement.tokens, "="
+    )
+    if expression is None:
+        return {
+            "cause": cause + _assign_to_identifiers_only(),
+            "suggest": _assign_to_identifiers_only(),
+        }
+    statement.location_markers = su.highlight_before_specified_token(
+        statement.bad_token, statement.tokens, "="
+    )
+    cause = _(
+        "On the left-hand side of an equal sign, you have a\n"
+        "conditional expression instead of the name of a variable.\n"
+        "    {expression} = ...\n"
+        "    {mark}\n"
+    ).format(
+        expression=expression,
+        mark=statement.location_markers[statement.first_token.start_row].strip(),
+    )
+
     return {
         "cause": cause + _assign_to_identifiers_only(),
         "suggest": _assign_to_identifiers_only(),
@@ -162,37 +187,38 @@ def assign_to_function_call(message: str = "", statement=None):
 
     hint = _assign_to_identifiers_only()
 
-    fn_call = statement.bad_token.string + "(...)"
-    line = statement.bad_line
-
-    if line.count("=") != 1 or line.count("(") != line.count(")"):
-        # we have something like  fn(a=1) = 2
-        # or fn(a) = 1 = 2, etc., and we cannot determine what is a function
-        # argument and what is the value assigned
-        value = _("some value")
+    fn_call = su.get_expression_before_specified_token(
+        statement.bad_token, statement.tokens, "="
+    )
+    if fn_call is None:
+        fn_call = statement.bad_token.string + "(...)"
         cause = _(
-            "You wrote an expression like\n\n"
-            "    {fn_call} = {value}\n\n"
-            "where `{fn_call}`, on the left-hand side of the equal sign, is\n"
-            "a function call and not the name of a variable.\n"
-        ).format(fn_call=fn_call, value=value)
-
-        return {"cause": cause + hint, "suggest": hint}
-
-    info = line.split("=")
-    fn_call = info[0].strip()
-    value = info[1].strip()
-    cause = _(
-        "You wrote the expression\n\n"
-        "    {fn_call} = {value}\n\n"
-        "where `{fn_call}`, on the left-hand side of the equal sign, either is\n"
-        "or includes a function call and is not simply the name of a variable.\n"
-    ).format(fn_call=fn_call, value=value)
+            "You wrote the expression\n\n"
+            "    {fn_call} = ...\n"
+            "where `{fn_call}`, on the left-hand side of the equal sign, either is\n"
+            "or includes a function call and is not simply the name of a variable.\n"
+        ).format(
+            fn_call=fn_call,
+        )
+    else:
+        statement.location_markers = su.highlight_before_specified_token(
+            statement.bad_token, statement.tokens, "="
+        )
+        cause = _(
+            "You wrote the expression\n\n"
+            "    {fn_call} = ...\n"
+            "    {mark}\n"
+            "where `{fn_call}`, on the left-hand side of the equal sign, either is\n"
+            "or includes a function call and is not simply the name of a variable.\n"
+        ).format(
+            fn_call=fn_call,
+            mark=statement.location_markers[statement.first_token.start_row].strip(),
+        )
     return {"cause": cause + hint, "suggest": hint}
 
 
 @add_python_message
-def assign_to_generator_expression(message: str = "", _statement=None):
+def assign_to_generator_expression(message: str = "", statement=None):
     if message not in (
         "can't assign to generator expression",  # Python 3.6, 3.7
         "cannot assign to generator expression",  # Python 3.8
@@ -203,6 +229,13 @@ def assign_to_generator_expression(message: str = "", _statement=None):
         "On the left-hand side of an equal sign, you have a\n"
         "generator expression instead of the name of a variable.\n"
     )
+    expression = su.get_expression_before_specified_token(
+        statement.bad_token, statement.tokens, "="
+    )
+    if expression is not None:
+        statement.location_markers = su.highlight_before_specified_token(
+            statement.bad_token, statement.tokens, "="
+        )
     hint = _assign_to_identifiers_only()
     return {"cause": cause + hint, "suggest": hint}
 
@@ -273,6 +306,13 @@ def assign_to_keyword(message: str = "", statement=None):
             "This is not allowed.\n"
             "\n"
         ).format(keyword=word)
+
+    # Ensure that the bad token is highlighted; for example: a.__debug__ = 1
+    if statement.bad_token != word:
+        for tok in statement.tokens:
+            if tok == word:
+                statement.location_markers = su.highlight_single_token(tok)
+                break
     return {"cause": cause, "suggest": hint}
 
 
@@ -307,6 +347,14 @@ def assign_to_literal(message: str = "", statement=None):
     ):
         return {}
 
+    expression = su.get_expression_before_specified_token(
+        statement.bad_token, statement.tokens, "="
+    )
+    if expression is not None:
+        statement.location_markers = su.highlight_before_specified_token(
+            statement.bad_token, statement.tokens, "="
+        )
+
     # This error can happen if we use a literal as an element of
     # a for loop; we take care of this case first.
     literal_in_for_loop = _assign_to_literal_in_for_loop(statement)
@@ -315,7 +363,7 @@ def assign_to_literal(message: str = "", statement=None):
 
     line = statement.bad_line.rstrip()
     parts = line.split("=")
-    if len(parts) == 2:
+    if len(parts) == 2:  # TODO: remove this limitation
         literal = parts[0].strip()
         name = parts[1].strip()
         if sys.version_info < (3, 8) and (
@@ -329,7 +377,10 @@ def assign_to_literal(message: str = "", statement=None):
             ).format(fstring=statement.bad_token)
             return {"cause": cause}
     else:
-        literal = None
+        if expression is not None:
+            literal = expression
+        else:
+            literal = None
         name = _("variable_name")
 
     if len(parts) == 2 and name.isidentifier():
@@ -352,12 +403,11 @@ def assign_to_literal(message: str = "", statement=None):
         of_type = _what_kind_of_literal("{1}")
     elif message == "cannot assign to dict display":
         of_type = _what_kind_of_literal("{1:2}")
-    else:
-        of_type = _what_kind_of_literal(literal)
-
-    if literal is None:
+    elif literal is None:
         literal = "..."
         of_type = ""
+    else:
+        of_type = _what_kind_of_literal(literal)
 
     cause = (
         _(
@@ -474,8 +524,8 @@ def assignment_cannot_rebind_inside_comprehension_inner_loop(
 
 
 @add_python_message
-def annotated_name_cannot_be_global(message: str = "", _statement=None):
-    pattern1 = re.compile(r"annotated name '(.)' can't be global")
+def annotated_name_cannot_be_global(message: str = "", statement=None):
+    pattern1 = re.compile(r"annotated name '(.*)' can't be global")
     match = re.search(pattern1, message)
     if not match:
         return {}
@@ -483,7 +533,9 @@ def annotated_name_cannot_be_global(message: str = "", _statement=None):
         "The object named `{name}` is defined with type annotation\n"
         "as a local variable. It cannot be declared to be a global variable.\n"
     ).format(name=match.group(1))
-
+    # Ensure that only the variable gets highlighted for consistency
+    # across all Python versions.
+    statement.location_markers = su.highlight_single_token(statement.bad_token)
     return {"cause": cause}
 
 
@@ -513,16 +565,16 @@ def both_nonlocal_and_global(message: str = "", statement=None):
 
 @add_python_message
 def bracket_was_expected(message: str = "", statement=None):
-    pattern = re.compile("'(.)' was never closed")  # new in Python 3.10
+    pattern = re.compile("'(.*)' was never closed")  # new in Python 3.10
     match = re.search(pattern, message)
     if not match:
         return {}
 
     cause = _("Python tells us that the {bracket} was never closed.\n").format(
-        bracket=syntax_utils.name_bracket(match.group(1))
+        bracket=su.name_bracket(match.group(1))
     )
     hint = _("The {bracket} was never closed.\n").format(
-        bracket=syntax_utils.name_bracket(match.group(1))
+        bracket=su.name_bracket(match.group(1))
     )
     rephrased_cause = statement_analyzer.unclosed_bracket(statement)
     if rephrased_cause:
@@ -712,8 +764,10 @@ def colon_expected(message: str = "", statement=None):
         )
         return {"cause": cause, "suggest": hint}
 
-    new_statement = fixers.modify_token(
-        statement.statement_tokens, statement.bad_token, append=":"
+    new_statement = fixers.replace_token(
+        statement.statement_tokens,
+        statement.bad_token,
+        statement.bad_token.string + ":",
     )
     if fixers.check_statement(new_statement):  # pragma: no cover
         debug_helper.log("New case for colon_expected.")
@@ -746,19 +800,44 @@ def continue_outside_loop(message: str = "", _statement=None):
 
 
 @add_python_message
-def duplicate_argument_in_function_definition(message: str = "", _statement=None):
+def duplicate_argument_in_function_definition(message: str = "", statement=None):
     if "duplicate argument" in message and "function definition" in message:
         name = message.split("'")[1]
         cause = _(
-            "You have defined a function repeating the keyword argument\n\n"
+            "You have defined a function repeating the argument\n\n"
             "    {name}\n"
-            "twice; each keyword argument should appear only once"
-            " in a function definition.\n"
+            "Each argument should appear only once in a function definition.\n"
         ).format(name=name)
+        locate_duplicate_arguments(statement, name)
         return {"cause": cause}
+
     return {}
 
 
+def locate_duplicate_arguments(statement, name):
+    # Make sure that we do not include the function name
+    begin_args = False
+    args = []
+    for token in statement.tokens:
+        if token == "(":
+            begin_args = True
+        if not begin_args:
+            continue
+        if token == name:
+            args.append(token)
+
+    markers = {}
+    for token in args:
+        if token.start_row not in markers:
+            markers[token.start_row] = " " * token.start_col + "^" * len(token.string)
+        else:
+            markers[token.start_row] += " " * (
+                token.start_col - prev_token.end_col  # noqa
+            ) + "^" * len(token.string)
+        prev_token = token  # noqa
+    statement.location_markers = markers
+
+
 @add_python_message
 def else_after_if(message: str = "", _statement=None):
     if message != "expected 'else' after 'if' expression":
@@ -787,26 +866,82 @@ def eof_unclosed_triple_quoted(message: str = "", _statement=None):
 
 @add_python_message
 def eol_while_scanning_string_literal(message: str = "", statement=None):
-    if (
+    if not (
         "EOL while scanning string literal" in message
-        or "unterminated string literal" in message  # Python 3.10
+        or "unterminated string literal" in message  # Python 3.10+
     ):
-        hint = _("Did you forget a closing quote?\n")
-        cause = _(
-            "You started writing a string with a single or double quote\n"
-            "but never ended the string with another quote on that line.\n"
-        )
-        # skipcq: PYL-R1714
-        # second if case for Python 3.10
-        if statement.prev_token == "\\" or statement.bad_line[-2] == "\\":
-            cause += _(
-                "Perhaps you meant to write the backslash character, `\\`\n"
-                "as the last character in the string and forgot that you\n"
-                "needed to escape it by writing two `\\` in a row.\n"
-            )
-            hint = _("Did you forget to escape a backslash character?\n")
+        return {}
 
-        return {"cause": cause, "suggest": hint}
+    # Default explanation
+    hint = _("Did you forget a closing quote?\n")
+    cause = _(
+        "You started writing a string with a single or double quote\n"
+        "but never ended the string with another quote on that line.\n"
+    )
+    # skipcq: PYL-R1714
+    # second if case for Python 3.10
+    if statement.prev_token == "\\" or statement.bad_line[-2] == "\\":
+        cause += _(
+            "Perhaps you meant to write the backslash character, `\\`\n"
+            "as the last character in the string and forgot that you\n"
+            "needed to escape it by writing two `\\` in a row.\n"
+        )
+        hint = _("Did you forget to escape a backslash character?\n")
+
+    # Perhaps we have an unescaped inner quote:
+    #  ... 'I don't care.'
+    # This gave "SyntaxError: invalid syntax" prior to Python 3.10.
+
+    previous_string = None
+    for tok in statement.tokens:
+        if tok == statement.bad_token:
+            break
+        if tok.is_string():
+            previous_string = tok
+
+    if previous_string is not None:
+        if previous_string.string[-1] == statement.bad_token:
+            new_tokens = []
+            for tok in statement.tokens:
+                if tok == previous_string:
+                    s = tok.string[:-1] + tok.string[-1].replace("'", "\\'").replace(
+                        '"', '\\"'
+                    )
+                    tok_copy = previous_string.copy()
+                    tok_copy.string = s
+                    new_tokens.append(tok_copy)
+                else:
+                    new_tokens.append(tok)
+            new_line = token_utils.untokenize(new_tokens)
+            if fixers.check_statement(new_line):
+                quote_position = previous_string.end_col
+                indent = len(statement.bad_line) - len(statement.bad_line.lstrip())
+                quote_position -= indent
+                mark = " " * (quote_position - 1) + "^^"
+
+                hint = _("Perhaps you forgot to escape a quote character.\n")
+                cause = _(
+                    "I suspect that you were trying to use a quote character inside a string\n"
+                    "that was enclosed in quotes of the same kind.\n"
+                    "Perhaps you should have escaped the inner quote character:\n\n"
+                    "    {new_line}\n"
+                    "    {mark}\n"
+                ).format(new_line=new_line, mark=mark)
+
+    return {"cause": cause, "suggest": hint}
+
+
+@add_python_message
+def expected_paren(message: str = "", statement=None):
+    if message != "expected '('":
+        return {}
+
+    if statement.first_token == "def" or (
+        statement.first_token == "async" and statement.tokens[1] == "def"
+    ):
+        cause = error_in_def.analyze_def_statement(statement)
+        if cause:
+            return cause
     return {}
 
 
@@ -968,6 +1103,20 @@ def from__future__not_defined(message: str = "", _statement=None):
     return {"cause": cause}
 
 
+@add_python_message
+def function_parameters_cannot_be_paren(message: str = "", statement=None):
+    if message != "Function parameters cannot be parenthesized":
+        return {}
+
+    if statement.first_token == "def" or (
+        statement.first_token == "async" and statement.tokens[1] == "def"
+    ):
+        cause = error_in_def.analyze_def_statement(statement)
+        if cause:
+            return cause
+    return {}
+
+
 @add_python_message
 def generator_expression_must_be_parenthesized(message: str = "", _statement=None):
     if "Generator expression must be parenthesized" not in message:
@@ -1007,14 +1156,12 @@ def invalid_character_in_identifier(message: str = "", statement=None):
         "which is not allowed.\n"
     ).format(bad_character=bad_character)
 
-    potential_cause = syntax_utils.identify_bad_quote_char(
-        bad_character, statement.bad_line
-    )
+    potential_cause = su.identify_bad_quote_char(bad_character, statement.bad_line)
     if potential_cause:
         potential_cause["cause"] = copy_paste + python_says + potential_cause["cause"]
         return potential_cause
 
-    potential_cause = syntax_utils.identify_unicode_fraction(bad_character)
+    potential_cause = su.identify_unicode_fraction(bad_character)
     if potential_cause:
         potential_cause["cause"] = copy_paste + python_says + potential_cause["cause"]
         return potential_cause
@@ -1047,16 +1194,14 @@ def invalid_decimal_literal(message: str = "", statement=None):
 
     if statement.next_token is not None and statement.bad_token.is_number():
         bad_character = statement.next_token.string
-        potential_cause = syntax_utils.identify_bad_math_symbol(
-            bad_character, statement.bad_line
-        )
+        potential_cause = su.identify_bad_math_symbol(bad_character, statement.bad_line)
         if potential_cause:
             potential_cause["cause"] = (
                 cause_prefix + "\n" + however + "\n\n" + potential_cause["cause"]
             )
             return potential_cause
 
-        potential_cause = syntax_utils.identify_unicode_fraction(bad_character)
+        potential_cause = su.identify_unicode_fraction(bad_character)
         if potential_cause:
             potential_cause["cause"] = (
                 cause_prefix + "\n" + however + "\n\n" + potential_cause["cause"]
@@ -1176,6 +1321,17 @@ def keyword_argument_repeated(message: str = "", statement=None):
     return {"cause": cause}
 
 
+@add_python_message
+def lambda_expression_parameters_cannot_be_paren(message: str = "", statement=None):
+    if message != "Lambda expression parameters cannot be parenthesized":
+        return {}
+
+    cause = statement_analyzer.lambda_with_paren(statement)
+    if cause:
+        return cause
+    return {}
+
+
 @add_python_message
 def leading_zeros_in_decimal_integers(message: str = "", statement=None):
     # Same as previous case but for Python 3.8+
@@ -1198,13 +1354,13 @@ def leading_zeros_in_decimal_integers(message: str = "", statement=None):
 @add_python_message
 def mismatched_parenthesis(message: str = "", statement=None):
     pattern1 = re.compile(
-        r"closing parenthesis '(.)' does not match opening parenthesis '(.)' on line (\d+)"
+        r"closing parenthesis '(.*)' does not match opening parenthesis '(.*)' on line (\d+)"
     )
     match = re.search(pattern1, message)
     if match is None:
         lineno = None
         pattern2 = re.compile(
-            r"closing parenthesis '(.)' does not match opening parenthesis '(.)'"
+            r"closing parenthesis '(.*)' does not match opening parenthesis '(.*)'"
         )
         match = re.search(pattern2, message)
         if match is None:
@@ -1252,7 +1408,7 @@ def named_arguments_must_follow_bare_star(message: str = "", _statement=None):
 
 
 @add_python_message
-def name_assigned_to_prior_global(message: str = "", _statement=None):
+def name_assigned_to_prior_global(message: str = "", statement=None):
     # something like: name 'p' is assigned to before global declaration
     if "is assigned to before global declaration" not in message:
         return {}
@@ -1262,11 +1418,23 @@ def name_assigned_to_prior_global(message: str = "", _statement=None):
         "You assigned a value to the variable `{name}`\n"
         "before declaring it as a global variable.\n"
     ).format(name=name)
+    # Highlight global and variable name individually
+    first, second = "", ""
+    for tok in statement.tokens:
+        if tok.string == "global":
+            first = tok
+        elif tok.string == name:
+            second = tok
+            break
+    if first and second:
+        statement.location_markers = su.highlight_two_tokens(
+            first, second, first_marker="-"
+        )
     return {"cause": cause}
 
 
 @add_python_message
-def name_assigned_to_prior_nonlocal(message: str = "", _statement=None):
+def name_assigned_to_prior_nonlocal(message: str = "", statement=None):
     # something like: name 'p' is assigned to before global declaration
     if "is assigned to before nonlocal declaration" not in message:
         return {}
@@ -1277,6 +1445,19 @@ def name_assigned_to_prior_nonlocal(message: str = "", _statement=None):
         "You assigned a value to the variable `{name}`\n"
         "before declaring it as a nonlocal variable.\n"
     ).format(name=name)
+    # Highlight nonlocal and variable name individually
+    first, second = "", ""
+    for tok in statement.tokens:
+        if tok.string == "nonlocal":
+            first = tok
+        elif tok.string == name:
+            second = tok
+            break
+
+    if first and second:
+        statement.location_markers = su.highlight_two_tokens(
+            first, second, first_marker="-"
+        )
     return {"cause": cause, "suggest": hint}
 
 
@@ -1319,7 +1500,7 @@ def name_is_parameter_and_nonlocal(message: str = "", _statement=None):
 
 
 @add_python_message
-def name_used_prior_global(message: str = "", _statement=None):
+def name_used_prior_global(message: str = "", statement=None):
     # something like: name 'p' is used prior to global declaration
     if "is used prior to global declaration" not in message:
         return {}
@@ -1328,11 +1509,24 @@ def name_used_prior_global(message: str = "", _statement=None):
     cause = _(
         "You used the variable `{name}`\nbefore declaring it as a global variable.\n"
     ).format(name=name)
+    # Highlight global and variable name individually
+    first, second = "", ""
+    for tok in statement.tokens:
+        if tok.string == "global":
+            first = tok
+        elif tok.string == name:
+            second = tok
+            break
+
+    if first and second:
+        statement.location_markers = su.highlight_two_tokens(
+            first, second, first_marker="-"
+        )
     return {"cause": cause}
 
 
 @add_python_message
-def name_used_prior_nonlocal(message: str = "", _statement=None):
+def name_used_prior_nonlocal(message: str = "", statement=None):
     # something like: name 'q' is used prior to nonlocal declaration
     if "is used prior to nonlocal declaration" not in message:
         return {}
@@ -1343,6 +1537,19 @@ def name_used_prior_nonlocal(message: str = "", _statement=None):
         "You used the variable `{name}`\n"
         "before declaring it as a nonlocal variable.\n"
     ).format(name=name)
+    # Highlight nonlocal and variable name individually
+    first, second = "", ""
+    for tok in statement.tokens:
+        if tok.string == "nonlocal":
+            first = tok
+        elif tok.string == name:
+            second = tok
+            break
+
+    if first and second:
+        statement.location_markers = su.highlight_two_tokens(
+            first, second, first_marker="-"
+        )
     return {"cause": cause, "suggest": hint}
 
 
@@ -1421,23 +1628,19 @@ def positional_argument_follows_keyword_arg(message: str = "", _statement=None):
 
 
 @add_python_message
-def python2_print(message: str = "", _statement=None):
+def python2_print(message: str = "", statement=None):
     if not message.startswith(
         "Missing parentheses in call to 'print'. Did you mean print("
     ):
         return {}
-    message = message[59:-2]
-    possible_statement = f"print({message})"
+
+    content = statement.entire_statement.replace("print", "", 1).strip()
+    possible_statement = f"print({content})"
     valid = fixers.check_statement(possible_statement)
-    if not valid:
-        if '"' not in message:
-            message = f'"{message}"'
-        elif "'" not in message:
-            message = f"'{message}'"
-        else:
-            message = "'...'"
-    if len(message) > 40:
-        message = message[0:25] + " ... "
+    if "\n" in content or len(content) > 40 or not valid:
+        message = "..."
+    else:
+        message = content
     cause = _(
         "Perhaps you need to type\n\n"
         "     print({message})\n\n"
@@ -1658,11 +1861,11 @@ def unicode_error(message: str = "", _statement=None):
 def unmatched_parenthesis(message: str = "", statement=None):
     # Python 3.8
     if message == "unmatched ')'":
-        bracket = syntax_utils.name_bracket(")")
+        bracket = su.name_bracket(")")
     elif message == "unmatched ']'":
-        bracket = syntax_utils.name_bracket("]")
+        bracket = su.name_bracket("]")
     elif message == "unmatched '}'":
-        bracket = syntax_utils.name_bracket("}")
+        bracket = su.name_bracket("}")
     else:
         return {}
     cause = _(
@@ -1727,14 +1930,14 @@ def you_found_it(message: str = "", statement=None):  # pragma: no cover
 
 
 @add_python_message
-def cannot_delete_something_else(message: str = "", statement=None):
+def cannot_delete_something_else(message: str = "", _statement=None):
     if not message.startswith("cannot delete"):
         return {}
     return {"cause": _can_only_delete()}
 
 
 @add_python_message
-def assign_to_others(message: str = "", statement=None):
+def assign_to_others(message: str = "", _statement=None):
     if not message.startswith("cannot assign to"):
         return {}
     hint = _assign_to_identifiers_only()
