@@ -8,7 +8,8 @@ import sys
 
 from .. import debug_helper, token_utils, utils
 from ..ft_gettext import current_lang, internal_error
-from . import error_in_def, fixers, syntax_utils
+from . import error_in_def, fixers
+from . import syntax_utils as su
 
 STATEMENT_ANALYZERS = []
 _ = current_lang.translate
@@ -70,7 +71,7 @@ def analyze_statement(statement):
     on which the error occurred."""
     if not statement.tokens:  # pragma: no cover
         debug_helper.log("Statement with no tokens in statement_analyser.py")
-        return {"cause": internal_error("No tokens")}
+        return {"cause": internal_error(Exception("No tokens"))}
 
     if statement.first_token == "def" or (
         statement.first_token == "async" and statement.tokens[1] == "def"
@@ -109,7 +110,7 @@ def mismatched_brackets(statement):
                 "The closing {bracket} on line {linenumber}"
                 " does not match anything.\n"
             ).format(
-                bracket=syntax_utils.name_bracket(statement.end_bracket),
+                bracket=su.name_bracket(statement.end_bracket),
                 linenumber=statement.end_bracket.start_row,
             )
             + source
@@ -124,24 +125,29 @@ def mismatched_brackets(statement):
 
     source = f"\n    {open_lineno}: {statement.source_lines[open_lineno - 1]}"
     shift = len(str(open_lineno)) + open_bracket.start_col + 6
+    statement.location_markers = {}
     if open_lineno == end_lineno:
         source += " " * shift + "^"
+        statement.location_markers[open_lineno] = " " * open_bracket.start_col + "^"
         shift = end_bracket.start_col - open_bracket.start_col - 1
         source += " " * shift + "^\n"
+        statement.location_markers[open_lineno] += " " * shift + "^"
     else:
         source += " " * shift + "^\n"
+        statement.location_markers[open_lineno] = " " * open_bracket.start_col + "^"
         source += f"    {end_lineno}: {statement.source_lines[end_lineno - 1]}"
         shift = len(str(end_lineno)) + end_bracket.start_col + 6
         source += " " * shift + "^\n"
+        statement.location_markers[end_lineno] = " " * end_bracket.start_col + "^"
 
     cause = (
         _(
             "The closing {bracket} on line {close_lineno} does not match "
             "the opening {open_bracket} on line {open_lineno}.\n"
         ).format(
-            bracket=syntax_utils.name_bracket(statement.end_bracket),
+            bracket=su.name_bracket(statement.end_bracket),
             close_lineno=statement.end_bracket.start_row,
-            open_bracket=syntax_utils.name_bracket(open_bracket),
+            open_bracket=su.name_bracket(open_bracket),
             open_lineno=open_bracket.start_row,
         )
         + source
@@ -162,22 +168,26 @@ def copy_pasted_code(statement):
         and first_token == statement.bad_token
         and statement.next_token == ">"
     ):
+        statement.location_markers = su.highlight_two_tokens(
+            first_token, statement.next_token
+        )
         cause = _(
             "It looks like you copy-pasted code from an interactive interpreter.\n"
             "The Python prompt, `>>>`, should not be included in your code.\n"
         )
         return {"cause": cause, "suggest": hint}
-    elif first_token == "...":
-        if statement.highlighted_tokens is not None:  # Python 3.10
-            is_ellipsis = statement.bad_token
-        else:
-            is_ellipsis = statement.prev_token
-        if is_ellipsis == first_token:
-            cause = _(
-                "It looks like you copy-pasted code from an interactive interpreter.\n"
-                "The Python prompt, `...`, should not be included in your code.\n"
-            )
-            return {"cause": cause, "suggest": hint}
+    elif (
+        statement.first_token == statement.bad_token
+        or statement.prev_token == statement.first_token
+    ):
+        statement.location_markers = su.highlight_two_tokens(
+            statement.first_token, statement.tokens[1], first_marker="-", between="-"
+        )
+        cause = _(
+            "It looks like you copy-pasted code from an interactive interpreter.\n"
+            "The Python prompt, `...`, should not be included in your code.\n"
+        )
+        return {"cause": cause, "suggest": hint}
     return {}  # pragma: no cover
 
 
@@ -264,8 +274,17 @@ def confused_elif(statement):
     # skipcq: PYL-R1714
     if statement.bad_token == "elseif" or statement.prev_token == "elseif":
         name = "elseif"
+        if statement.first_token == "elseif":
+            statement.location_markers = su.highlight_single_token(
+                statement.first_token
+            )
+        else:
+            debug_helper.log("problem in confused_elif")
     elif statement.bad_token == "if" and statement.prev_token == "else":
         name = "else if"
+        statement.location_markers = su.highlight_two_tokens(
+            statement.prev_token, statement.bad_token, between="-"
+        )
     if name:
         hint = _("Perhaps you meant to write `elif`.\n")
         cause = _(
@@ -284,24 +303,25 @@ def import_from(statement):
 
     function = statement.prev_token
     module = statement.next_token
-
+    statement.location_markers = su.highlight_two_tokens(
+        statement.tokens[0], statement.bad_token
+    )
     hint = _("Did you mean `from {module} import {function}`?\n").format(
         module=module, function=function
     )
     cause = _(
         "You wrote something like\n\n"
-        "    import {function} from {module}\n"
+        "    import {function} from {module}\n\n"
         "instead of\n\n"
         "    from {module} import {function}\n\n"
-        "\n"
     ).format(module=module, function=function)
     return {"cause": cause, "suggest": hint}
 
 
 @add_statement_analyzer
 def misplaced_quote(statement):
-    """This looks for a misplaced quote, something like
-       info = 'don't ...
+    """This looks for a single misplaced quote, something like
+       info = 'I don't mind.'
 
     The clue we are looking for is a STRING token ('don')
     followed by something else than a string.
@@ -309,18 +329,35 @@ def misplaced_quote(statement):
     if not statement.prev_token.is_string():
         return {}
 
-    bad_token = statement.bad_token
-    if bad_token.is_identifier():
-        hint = _("Perhaps you misplaced a quote.\n")
-        cause = _(
-            "There appears to be a Python identifier (variable name)\n"
-            "immediately following a string.\n"
-            "I suspect that you were trying to use a quote inside a string\n"
-            "that was enclosed in quotes of the same kind.\n"
-        )
-        return {"cause": cause, "suggest": hint}
+    # Create a new line escaping an inner quote for the entire line.
+    new_tokens = []
+    for tok in statement.tokens:
+        if tok == statement.prev_token:
+            s = tok.string[:-1] + tok.string[-1].replace("'", "\\'").replace('"', '\\"')
+            tok_copy = statement.prev_token.copy()
+            tok_copy.string = s
+            new_tokens.append(tok_copy)
+        else:
+            new_tokens.append(tok)
 
-    return {}
+    new_line = token_utils.untokenize(new_tokens)
+    if not fixers.check_statement(new_line):
+        return {}
+
+    quote_position = statement.prev_token.end_col
+    indent = len(statement.bad_line) - len(statement.bad_line.lstrip())
+    quote_position -= indent
+    mark = " " * (quote_position - 1) + "^^"
+
+    hint = _("Perhaps you forgot to escape a quote character.\n")
+    cause = _(
+        "I suspect that you were trying to use a quote character inside a string\n"
+        "that was enclosed in quotes of the same kind.\n"
+        "Perhaps you should have escaped the inner quote character:\n\n"
+        "    {new_line}\n"
+        "    {mark}\n"
+    ).format(new_line=new_line, mark=mark)
+    return {"cause": cause, "suggest": hint}
 
 
 @add_statement_analyzer
@@ -394,6 +431,7 @@ def inverted_operators(statement):
         correct=correct,
         wrong=first.string + second.string,
     )
+    statement.location_markers = su.highlight_two_tokens(first, second)
 
     new_statement = fixers.replace_two_tokens(
         statement.statement_tokens,
@@ -415,6 +453,10 @@ def consecutive_operators(statement):
     if not (is_op(statement.bad_token) and is_op(statement.prev_token)):
         return {}
 
+    statement.location_markers = su.highlight_two_tokens(
+        statement.prev_token, statement.bad_token
+    )
+
     if statement.bad_token == "=" and statement.prev_token == "==":
         cause = _(
             "You wrote three equal signs in a row which is allowed in some\n"
@@ -475,6 +517,7 @@ def _walrus_instead_of_equal_39(statement):
 
 @add_statement_analyzer
 def walrus_instead_of_equal(statement):
+    # TODO: check this
     if (3, 8) < sys.version_info < (3, 10):
         bad_token = _walrus_instead_of_equal_39(statement)
         if bad_token is None:
@@ -556,32 +599,22 @@ def print_as_statement(statement):
     ):
         return {}
 
+    content = statement.entire_statement.replace("print", "", 1).strip()
+    possible_statement = f"print({content})"
+    valid = fixers.check_statement(possible_statement)
+    if "\n" in content or len(content) > 40 or not valid:
+        message = "..."
+    else:
+        message = content
     cause = _(
+        "Perhaps you need to type\n\n"
+        "     print({message})\n\n"
         "In older version of Python, `print` was a keyword.\n"
         "Now, `print` is a function; you need to use parentheses to call it.\n"
-    )
-    bad_line = statement.bad_line
-    valid = True
-    if bad_line.count("(") == bad_line.count(")"):
-        new_line = bad_line.replace("print", "", 1).strip()
-        possible_statement = f"print({new_line})"
-        valid = fixers.check_statement(possible_statement)
-        if not valid:
-            if '"' not in new_line:
-                new_line = f'"{new_line}"'
-            elif "'" not in new_line:
-                new_line = f"'{new_line}'"
-            else:
-                new_line = "..."
-        if len(new_line) > 30:
-            parts = new_line.split(" ")
-            new_line = parts[0] + " ... " + parts[-1]
-        new_line = "print(" + new_line + ")"
-    else:
-        new_line = "print(...)"
-    hint = _("Did you mean `{new_line}`?\n").format(new_line=new_line)
+    ).format(message=message)
     if not valid:
         cause += _("Note that arguments of `print` must be separated by commas.\n")
+    hint = _("Did you mean `print({message})`?\n").format(message=message)
     return {"cause": cause, "suggest": hint}
 
 
@@ -684,8 +717,10 @@ def missing_colon(statement):
     ):
         return {}
 
-    new_statement = fixers.modify_token(
-        statement.statement_tokens, statement.bad_token, append=":"
+    new_statement = fixers.replace_token(
+        statement.statement_tokens,
+        statement.bad_token,
+        statement.bad_token.string + ":",
     )
     if not fixers.check_statement(new_statement):
         return {}
@@ -693,6 +728,7 @@ def missing_colon(statement):
     name = statement.first_token
 
     hint = _("Did you forget a colon `:`?\n")
+    statement.location_markers = su.highlight_added_token(statement.bad_token, ":")
 
     if name.string in ("for", "while"):
         cause = _(
@@ -712,7 +748,7 @@ def missing_colon(statement):
 
 @add_statement_analyzer
 def semi_colon_instead_of_comma(statement):
-    """Writing a semi colon as a typo"""
+    """Writing a semicolon as a typo"""
     if statement.bad_token != ";":
         return {}
 
@@ -720,11 +756,11 @@ def semi_colon_instead_of_comma(statement):
         statement.statement_tokens, statement.bad_token, ","
     )
     if fixers.check_statement(new_statement):
-        cause = _("You wrote a semi-colon, `;`, where a comma was expected.\n")
+        cause = _("You wrote a semicolon, `;`, where a comma was expected.\n")
         hint = _("Did you mean to write a comma?\n")
         return {"cause": cause, "suggest": hint}
 
-    # perhaps used multiple semi-colons instead of comma
+    # perhaps used multiple semicolons instead of comma
     if statement.last_token == ";":
         new_statement = fixers.replace_token(
             statement.statement_tokens, statement.last_token, ""
@@ -740,13 +776,30 @@ def semi_colon_instead_of_comma(statement):
             break
         new_statement = fixers.replace_token(tokens, tok, ",")
     if fixers.check_statement(new_statement):
-        cause = _("You wrote semi-colons, `;`, where commas were expected.\n")
+        cause = _("You wrote semicolons, `;`, where commas were expected.\n")
         hint = _("Did you mean to write commas?\n")
         return {"cause": cause, "suggest": hint}
 
     return {}  # pragma: no cover
 
 
+@add_statement_analyzer
+def semi_colon_instead_of_colon(statement):
+    """Writing a semicolon as a typo"""
+    if statement.bad_token != ";":
+        return {}
+
+    new_statement = fixers.replace_token(
+        statement.statement_tokens, statement.bad_token, ":"
+    )
+    if fixers.check_statement(new_statement):
+        cause = _("You wrote a semicolon, `;`, where a colon was expected.\n")
+        hint = _("Did you mean to write a colon?\n")
+        return {"cause": cause, "suggest": hint}
+
+    return {}  # pragma: no cover
+
+
 @add_statement_analyzer
 def invalid_hexadecimal(statement):
     """Identifies problem caused by invalid character in an hexadecimal number."""
@@ -829,8 +882,10 @@ def invalid_name(statement):
             "[Note: `{first} * {second}` would also be valid\n"
             "since `{first}` is a complex number.]\n"
         ).format(first=first, second=second)
-        second.string = first.string[-1] + second.string
-        first.string = first.string[:-1]
+        # Do not modify the token themselves; reuse names and create pure strings
+        # otherwise the location shown by where() will be incorrect.
+        second = first.string[-1] + second.string
+        first = first.string[:-1]
     else:
         note = ""
 
@@ -889,10 +944,13 @@ def assign_to_a_keyword(statement):
     )
     if statement.bad_token == "=" and statement.prev_token.is_keyword():
         cause = possible_cause.format(keyword=statement.prev_token)
+        bad_token = statement.prev_token
     elif statement.bad_token.is_keyword() and statement.next_token == "=":
+        bad_token = statement.bad_token
         cause = possible_cause.format(keyword=statement.bad_token)
     else:
         return {}
+    statement.location_markers = su.highlight_single_token(bad_token)
     return {"cause": cause, "suggest": hint}
 
 
@@ -929,7 +987,7 @@ def lambda_with_paren(statement):
 
 @add_statement_analyzer
 def wrong_type_declaration(statement):
-    if statement.highlighted_tokens:  # Python 3.10
+    if statement.highlighted_tokens and len(statement.highlighted_tokens) > 1:
         bad_token = statement.next_token
         prev_token = statement.bad_token
     else:
@@ -976,8 +1034,8 @@ def missing_comma_before_string_in_dict(statement):
     """Special case where keys and values in a dict are strings which are
     not separated by commas."""
 
-    # This is a bit of an usual case as the error occurred due to a forgotten comma
-    # two tokens before the token flagged by Python.
+    # This is a bit of an unusual case as the error occurred due to
+    # a forgotten comma two tokens before the token flagged by Python.
     if not (
         statement.begin_brackets
         and statement.begin_brackets[-1] == "{"
@@ -987,8 +1045,10 @@ def missing_comma_before_string_in_dict(statement):
     ):
         return {}
 
-    new_statement = fixers.modify_token(
-        statement.statement_tokens, statement.prev_token, prepend=","
+    before_prev = statement.tokens[statement.bad_token_index - 2]
+
+    new_statement = fixers.replace_token(
+        statement.statement_tokens, before_prev, before_prev.string + ","
     )
     if not fixers.check_statement(new_statement):
         return {}
@@ -997,8 +1057,13 @@ def missing_comma_before_string_in_dict(statement):
         "I am guessing that you forgot a comma between two strings\n"
         "when defining a dict.\n\n"
     ).format(kwd=statement.bad_token)
-    new_statement = fixers.modify_token(
-        statement.statement_tokens, statement.prev_token, prepend=" «,» "
+    mark = su.highlight_added_token(before_prev, ",")
+
+    new_statement = fixers.replace_token(
+        statement.statement_tokens, before_prev, before_prev.string + ","
+    )
+    new_statement = su.add_mark_to_new_statement(
+        statement, new_statement, mark[before_prev.start_row]
     )
     cause += "```\n" + new_statement + "\n```"
     hint = _("Did you forget a comma?\n")
@@ -1015,7 +1080,8 @@ def missing_in_with_for(statement):
     """
     index = statement.bad_token_index
     bad_token = statement.bad_token
-    if statement.highlighted_tokens:  # Python 3.10 may highlight two tokens
+    # Python 3.10 may highlight two tokens
+    if statement.highlighted_tokens and len(statement.highlighted_tokens) > 1:
         index += 1
         bad_token = statement.next_token
 
@@ -1188,13 +1254,13 @@ def space_in_variable_name(statement):
     # my name = André
     bad_token = statement.bad_token
     prev_token = statement.prev_token
-    if statement.highlighted_tokens:  # Python 3.10
+    if statement.highlighted_tokens and len(statement.highlighted_tokens) == 2:
         bad_token = statement.next_token
         prev_token = statement.bad_token
 
     if not (
         bad_token.is_identifier()
-        and prev_token.is_identifier
+        and prev_token.is_identifier()
         and prev_token is statement.first_token
     ):
         return {}
@@ -1203,7 +1269,10 @@ def space_in_variable_name(statement):
     for tok in statement.tokens:
         if tok == "=":
             # Note: there could be other errors
-            cause = _("You cannot have spaces in identifiers (variable names).\n")
+            cause = _(
+                "You cannot have spaces in identifiers (variable names).\n"
+                "Perhaps you meant `{name}`?\n"
+            ).format(name="_".join(first_tokens))
             hint = _("Did you mean `{name}`?\n").format(name="_".join(first_tokens))
             return {"cause": cause, "suggest": hint}
 
@@ -1239,9 +1308,8 @@ def _add_comma_or_operator(tokens, tok, comma_first=True):
     for operator in operators:
         if operator == " in " and results:
             break
-        new_statement = fixers.modify_token(tokens, tok, append=operator)
+        new_statement = fixers.replace_token(tokens, tok, tok.string + operator)
         if fixers.check_statement(new_statement):
-
             results.append((operator.strip(), new_statement))
     return results
 
@@ -1252,20 +1320,20 @@ def _comma_first_cause(bracket):
             "It is possible that you "
             "forgot a comma between items in a tuple, \n"
             "or between function arguments, \n"
-            "before the position indicated by ^.\n"
+            "at the position indicated by ^.\n"
         )
 
     if bracket == "[":
         return _(
             "It is possible that you "
             "forgot a comma between items in a list\n"
-            "before the position indicated by ^.\n"
+            "at the position indicated by ^.\n"
         )
 
     return _(
         "It is possible that you "
         "forgot a comma between items in a set or dict\n"
-        "before the position indicated by ^.\n"
+        "at the position indicated by ^.\n"
     )
 
 
@@ -1301,34 +1369,61 @@ def missing_comma_or_operator(statement):
         comma_first = False
         comma_first_cause = ""
 
-    results = _add_comma_or_operator(
+    new_statements = _add_comma_or_operator(
         statement.statement_tokens, prev_token, comma_first=comma_first
     )
-    if not results:
+    if not new_statements:
         return {}
 
-    if len(results) == 1 or statement.first_token.string in ["def", "async", "class"]:
-        operator, line = results[0]
-        # reducing multiple spaces to single space for nicer display
-        temp = line.split(" ")
-        temp = [x for x in temp if x]
-        line = " ".join(temp)
+    if prev_token.start_row == bad_token.start_row:
+        statement.location_markers = su.highlight_two_tokens(
+            prev_token, bad_token, between="^"
+        )
+    else:
+        statement.location_markers = su.highlight_two_tokens(
+            prev_token, bad_token, first_marker="^", between="-->"
+        )
+
+    # TODO: fix the cases with def/async/class
+    if len(new_statements) == 1 or statement.first_token.string in [
+        "def",
+        "async",
+        "class",
+    ]:
+        operator, line = new_statements[0]
+        if "\n" in line:
+            mark = su.highlight_added_token(prev_token, "^")
+            line = su.add_mark_to_new_statement(
+                statement, line, mark[prev_token.start_row]
+            )
+            # lines = line.split("\n")
+            # lines = ["    " + line_ for line_ in lines]
+            # line = "    \n".join(lines)
+        else:
+            # reducing multiple spaces on single line to single space for nicer display
+            temp = line.split(" ")
+            temp = [x for x in temp if x]
+            line = " ".join(temp)
+            mark = su.highlight_added_token(prev_token, "^")
+            line = su.add_mark_to_new_statement(
+                statement, line, mark[prev_token.start_row]
+            )
         if "," in operator:
             hint = _("Did you forget a comma?\n")
             cause = possible_cause + comma_first_cause
-            cause += _("Perhaps you meant\n\n    {line}\n").format(line=line)
+            cause += _("Perhaps you meant{line}").format(line=utils.to_code_block(line))
         else:
             hint = _("Did you mean `{line}`?\n").format(line=line)
             cause = possible_cause
     else:
-        operators = [operator for operator, line in results]
-        lines = [line for operator, line in results]
+        operators = [operator for operator, line in new_statements]
+        lines = [line for operator, line in new_statements]
         hint = _("Did you forget something between `{first}` and `{second}`?\n").format(
             first=prev_token, second=bad_token
         )
 
         # The list of operators might include a comma; it is better to separate
-        # items by semi-colons
+        # items by semicolons
         if comma_first:
             operators = operators[1:]
             operators = utils.list_to_string(operators)
@@ -1581,7 +1676,7 @@ def unclosed_bracket(statement):
     bracket = statement.begin_brackets[-1]
     # not statement_brackets -> all brackets are closed
     # bad_token match open bracket: problem is not with unclosed bracket
-    if not statement.statement_brackets and syntax_utils.matching_brackets(
+    if not statement.statement_brackets and su.matching_brackets(
         bracket, statement.bad_token
     ):
         return {}
@@ -1597,7 +1692,7 @@ def unclosed_bracket(statement):
     linenumber = bracket.start_row
     start_col = bracket.start_col
 
-    bracket_name = syntax_utils.name_bracket(bracket)
+    bracket_name = su.name_bracket(bracket)
     source = f"\n    {linenumber}: {statement.source_lines[linenumber - 1]}"
     shift = len(str(linenumber)) + start_col + 6
     source += " " * shift + "^\n"
